#ifndef CONTEXTOTFUNCTIONVIEWER_H
#define CONTEXTOTFUNCTIONVIEWER_H


#include <qwidget.h>
#include <qevent.h>
#include <qpainter.h>

#include "vtkPiecewiseFunction.h"
#include "GradientFunctionStruct.h"


//! Class contextOTFunctionViewer
/*!
    Diese Klasse ist eine Subklasse von QWidget. Sie erzeugt ein Widget, welches die
	Context Opacity Transfer Function darstellt und in dem die Context Opaciity Transfer Function auch
	ver&auml;nedert werden kann.
*/
class contextOTFunctionViewer : public QWidget
{

public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen.
	*/
	contextOTFunctionViewer(QWidget *parent, const char *name);
	//! Destruktor
	~contextOTFunctionViewer();
	//! Methode resetOpacityFunction
	/*!
		Setzt die Context Opacity Function zur&uuml;ck.
	*/
	void resetOpacityFunction();
	//! Methode setOpacityFunction
	/*!
		Setzt die Context Opacity Function auf den &uuml;bergebenen Wert.
	*/
	void setOpacityFunction(vtkPiecewiseFunction *piecewiseFunction, float range);
	//! Methode getOpacityFunction
	/*!
		Gibt die Context Opacity Function zur&uuml;ck.
	*/
	void getOpacityFunction(vtkPiecewiseFunction *piecewiseFunction, float range);
	//! Methode getChanged
	/*!
		Ist true, wenn Werte im Context Opacity Viewer ge&auml;ndert wurden.
	*/
	bool getChanged();
	//! Methode resetChanged
	/*!
		Setzt boolsche Variable, welche &Auml;nderungen angibt, auf false.
	*/
	void resetChanged();
	//! Methode setHistogram
	/*!
		Setzt das Histogramm auf die &uuml;bergebenen Werte.
	*/
	void setHistogram(unsigned int *histo);

protected:
    void paintEvent(QPaintEvent *event);
	void mousePressEvent(QMouseEvent *event);
	void mouseReleaseEvent(QMouseEvent *event);
	void mouseDoubleClickEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);

private:
	void paintHistogram(QPainter *paint);
	void paintCubes(QPainter *paint);
	void paintLines(QPainter *paint);
	void insertPoint(float x, float y);
	void deletePoint(int index);
	int testIntersection(int x, int y, int index);

	gradientFunc opacityFunction [20];
	unsigned int histogram[256];
	int points;
	bool mousePressed;
	bool movingX;
	int intersectionIndex;
	bool changed;

};

#endif