
#include <qlineedit.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qcheckbox.h>


#include "DOFAnimationEditorImpl.h"
#include "focalEditorImpl.h"



DOFAnimationEditorImpl::DOFAnimationEditorImpl(QWidget *parent, const char * name, int dim [3], float orient [3], double focPoint [3]) : DOFAnimationEditor (parent, name)
{
    for (int i = 0; i < 3; i++)
    {
		dimension[i] = dim[i];
		focalPoint[i] = focPoint[i];
		orientation[i] = orient[i];
    }
    startLabel->setText("0");
    endLabel->setText("0");
    frameLabel->setText("2");
	samplePointsLabel->setText("2");
    numberPoints = 0;
    rendering = false;
}


void DOFAnimationEditorImpl::cancelPressed()
{
    this->hide();
}


void DOFAnimationEditorImpl::startPressed()
{
    if (numberPoints >= 2)
    {
		rendering = true;
		this->hide();
    }
    else
    {
		QMessageBox::warning(this, "Depth of Field", "Please choose at least two Focal Points!");
    }
}


void DOFAnimationEditorImpl::addPointPressed()
{
    if (numberPoints <= 20)
    {
		focalEditorImpl *focEditor = new focalEditorImpl(this, "Renderer - Set Focal Point", dimension, orientation, focalPoint);
		focEditor->exec();
		if (focEditor->okay)
		{
			double *fP = new double [3];
			fP = focEditor->getFocalPoint();
			focalArray[numberPoints][0] = fP[0];
			focalArray[numberPoints][1] = fP[1];
			focalArray[numberPoints][2] = fP[2];
			pointBox->insertItem("x: "+QString::number(fP[0])+"  y: "+QString::number(fP[1])+"  z: "+QString::number(fP[2]));
			pointBox->setSelected(numberPoints, true);
			numberPoints++;
			//deletePointButton->setEnabled(true);
		}
		delete focEditor;
    }
    else
    {
		QMessageBox::warning(this, "Depth of Field", "Maximum of Points is reached!");
    }
}


void DOFAnimationEditorImpl::deletePointPressed()
{
    int new_index, index = pointBox->currentItem();
    if (index >= 0)
    {
		if (index == (int)(pointBox->count()) - 1)
		{
			new_index = index - 1;
		}
		else
		{
			new_index = index;
		} 
		pointBox->removeItem(index);
		pointBox->setSelected(new_index, true);
		for (int i = index; i < numberPoints-1; i++)
		{
			focalArray[i][0] = focalArray[i+1][0];
			focalArray[i][1] = focalArray[i+1][1];
			focalArray[i][2] = focalArray[i+1][2];
		}
		numberPoints = numberPoints-1;
    }
}


void DOFAnimationEditorImpl::startChanged(int value)
{
    start = (float)value/10;
    startLabel->setText(QString::number (start));
}


void DOFAnimationEditorImpl::endChanged(int value)
{
    end = (float)value/10;
    endLabel->setText(QString::number(end));
}


void DOFAnimationEditorImpl::framesChanged(int value)
{
    frames = value;
    frameLabel->setText(QString::number(frames));
}


void DOFAnimationEditorImpl::samplePointsChanged(int value)
{
	samplePoints = value;
	samplePointsLabel->setText(QString::number(samplePoints));
}


float DOFAnimationEditorImpl::getStart()
{
    return start;
}


float DOFAnimationEditorImpl::getEnd()
{
    return end;
}


int DOFAnimationEditorImpl::getFrames()
{
    return frames;
}


int DOFAnimationEditorImpl::getSamplePoints()
{
	return samplePoints;
}


bool DOFAnimationEditorImpl::getFixedPoint()
{
	return fixedPoint->isChecked();
}
