
#include <qpushbutton.h>
#include <qframe.h>
#include <qslider.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qkeycode.h>
#include <qaction.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qregexp.h>

#include "mainWidgetImpl.h"
//#include "glbox.h"
#include "dimdialog.h"
#include "reader.h"
#include "transferFunctionEditorImpl.h"


DOFMainWindowImpl::DOFMainWindowImpl (QWidget* parent, const char* name, WFlags f) : dofmainwindow (parent, name, f)
{
}


DOFMainWindowImpl::~DOFMainWindowImpl ()
{
    VTKViewer->Delete();
}


void GLObjectWindow::openFile()
{
    bool flag = false, dimensionRead = false;
    QString filename = QFileDialog::getOpenFileName(
			    QString::null, "Volume (*.dat *.raw)", this,
			    "file open", "File Open" );
    
    if ( !filename.isEmpty() )
    {
	int* dimension = readDimension (filename);
	
	//  read Dimension from filename or from DimDialog, if the filename contain no Dimension
	if (dimension == NULL)
	{
	    DimDialog *dim = new DimDialog (this);
	    dim->exec();
	    if (dim->flag)
	    {
		dimension = dim->getValues();
		dimensionRead =true;
	    }
	    delete dim;
	}
	else
	{
	    dimensionRead = true;
	}
	
	// if Dimensions could be read
	if (dimensionRead)
	{
	    flag = true;
	    Reader *imageReader = new Reader (filename, dimension);
	    if (imageReader->getErrorReport() == 0)
	    {
		VTKViewer->setRenderer (imageReader->getData());
		//transferFunctionEditorImpl *editor = new transferFunctionEditorImpl (this, "editor", imageReader->getHistogram());
		//editor->exec();
		//delete editor;
		transfer_funcViewer->setHistogram (imageReader->getHistogram);
	    }
	    else
	    {
		if (imageReader->getErrorReport() == 1)
		{
		    QMessageBox::warning( this, "Depth of Field", "Could not open file." );
		}
		else
		{
		    QMessageBox::warning( this, "Depth of Field", "Wrong File Dimensions." );
		}
	    }
	}
    }
    else         // if no File selected
    {
	flag = true;
    }
    
    // if no Data loaded while no Dimensions where found
    if (flag == false)
    {
	QMessageBox::information( this, "Depth of Field", "Can not read file" );
    }
}


int* GLObjectWindow::readDimension (QString filename)
{
    int* dimension = new int [3];
    bool ok;

    QRegExp rxp1 = QRegExp ("[0-9]+x[0-9]+x[0-9]", TRUE, FALSE);
    QRegExp rxp2 = QRegExp ("[^x0-9]", TRUE, FALSE);
    
    QString dim = filename.section (rxp1, 1, 1, QString::SectionIncludeLeadingSep);
    dim = dim.section (rxp2, 0, 0);
    if (dim.length() < 5)
    {
	return NULL;
    }
    if (dim.contains ("x") > 2)
    {
	dim = dim.section ("x", 0, 2);
    }
    QString x = dim.section ("x", 0, 0);
    QString y = dim.section ("x", 1, 1);
    QString z = dim.section ("x", 2, 2);
    
    dimension[0] = x.toInt(&ok, 10);
    dimension[1] = y.toInt(&ok, 10);
    dimension[2] = z.toInt(&ok, 10);
    
    /*cout << "Filename:  " << filename << "\n";
    cout << "Dimension begin: " << begin << "\n";
    cout << "Dimension end:   " << end << "\n";
    cout << "Dimension:   " << dim << "\n";
    cout << "x:   " << x << "\n";
    cout << "y:   " << y << "\n";
    cout << "z:   " << z << "\n";*/
    
    return dimension;
}
