#ifndef DITHANIM_NEWPOINTIMPL_h
#define DITHANIM_NEWPOINTIMPL_h

#include <qwidget.h>

#include "DithAnim_newPoint.h"
#include "DithAnim_struct.h"



//! Class DithAnim_newPointImpl
/*!
    Diese Klasse ist abgeleitet von der Klasse DithAnim_newPoint. In ihr sind die 
	Methoden fr die Actions im Dialog ausprogrammiert.
*/
class DithAnim_newPointImpl : public DithAnim_newPoint
{
    Q_OBJECT;
    
public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen. newPoint ist eine Struktur, 
		welche die Werte des aktuellen Animations Punkts enth&auml;t. oldPoint enth&auml;lt die
		Werte des Vorg&auml;nger.
	*/
    DithAnim_newPointImpl(QWidget* parent, const char* name, DithAnim_struct newPoint, DithAnim_struct oldPoint);
    //! Destruktor
	~DithAnim_newPointImpl();
	//! Methode getExitStatus
	/*!
		Gibt true zur&uuml;ck, wenn der Dialog mit OK beendet wurde.
	*/
	bool getExitStatus();
	//! Methode getDithAnimStruct
	/*!
		Gibt die neuen Werte des Animationspunkts in einer Struktur zur&uuml;ck.
	*/
	DithAnim_struct getDithAnimStruct();
    
public slots:
    //! Slot xRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die x-Rotation bewegt wird.
	*/
    void xRotSliderChanged(int value);
	//! Slot yRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die y-Rotation bewegt wird.
	*/
	void yRotSliderChanged(int value);
	//! Slot zRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die z-Rotation bewegt wird.
	*/
	void zRotSliderChanged(int value);
	//! Slot zoomSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r den Zoom bewegt wird.
	*/
	void zoomSliderChanged(int value);
	//! Slot okayButtonClicked
	/*!
		Wird aufgerufen wenn der OK Button geclickt wird.
	*/
	void okayButtonClicked();
	//! Slot samplePointsSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Anzahl der Frames bewegt wird.
	*/
	void samplePointsSliderChanged(int value);
	//! Slot xRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die x-Achse
		geclickt wird.
	*/
	void xRotClockwiseClicked(bool status);
	//! Slot yRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die y-Achse
		geclickt wird.
	*/
	void yRotClockwiseClicked(bool status);
	//! Slot zRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die z-Achse
		geclickt wird.
	*/
	void zRotClockwiseClicked(bool status);
	//! Slot distanceSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Distanz der Dithering Quader bewegt wird.
	*/
	void distanceSliderChanged(int value);
	//! Slot sizeSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Gr&ouml;sse der Dithering Quader bewegt wird.
	*/
	void sizeSliderChanged(int value);
    
private:
	bool getValues();
	void setValues();

	DithAnim_struct anim;
	DithAnim_struct animRef;
	bool exitStatus;
};


#endif