#include <qlistbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>

#include "DithAnimationEditorImpl.h"
#include "DithAnim_startPointImpl.h"
#include "DithAnim_newPointImpl.h"



DithAnimationEditorImpl::DithAnimationEditorImpl(QWidget* parent, const char* name, DithAnim_struct startPoint) : DithAnimationEditor(parent, name)
{
	exitStatus = false;
	anim = new DithAnim_struct [30];
	anim[0] = startPoint;
	number = 1;
	frames = 0;
	frameCounter->setText("0");
}


DithAnimationEditorImpl::~DithAnimationEditorImpl()
{
}


bool DithAnimationEditorImpl::getExitStatus()
{
	return exitStatus;
}


DithAnim_struct *DithAnimationEditorImpl::getDithAnimStruct()
{
	return anim;
}


int DithAnimationEditorImpl::getNumbers()
{
	return number;
}


void DithAnimationEditorImpl::deletePointButtonClicked()
{
	int pos = pointBox->currentItem();
	if (pos >= 0)
	{
		for (int i = pos+1; i < number-1; i++)
		{
			anim[i] = anim[i+1];
		}
		number = number-1;
		pointBox->removeItem(pos);
		if (pos+1 < number)
			pointBox->setCurrentItem(pos);
		else
			pointBox->setCurrentItem(pos-1);
		frames = 0;
		for (i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
	}
}


void DithAnimationEditorImpl::editPointButtonClicked()
{
	int pos = pointBox->currentItem()+1;
	DithAnim_newPointImpl *editPoint = new DithAnim_newPointImpl(this, "Edit Point", anim[pos], anim[pos-1]);
	editPoint->exec();
	if (editPoint->getExitStatus())
	{
		anim[pos] = editPoint->getDithAnimStruct();
		frames = 0;
		for (int i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
		QString frameString = "[" + QString::number(anim[pos].samples) + " frames]";
		pointBox->changeItem(frameString, pos-1);
	}
	delete editPoint;
}


void DithAnimationEditorImpl::newPointButtonClicked()
{
	if (number <= 30)
	{
		DithAnim_newPointImpl *newPoint = new DithAnim_newPointImpl(this, "Edit Point", anim[number-1], anim[number-1]);
		newPoint->exec();
		if (newPoint->getExitStatus())
		{
			anim[number] = newPoint->getDithAnimStruct();
			number = number+1;
			QString frameString = "[" + QString::number(anim[number-1].samples) + " frames]";
			pointBox->insertItem(frameString, -1);
			frames = 0;
			for (int i = 1; i < number; i++)
				frames += anim[i].samples; 
			frameCounter->setText(QString::number(frames));
		}
		delete newPoint;
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "Maximum Point Count is reached");
}


void DithAnimationEditorImpl::startPointButtonClicked()
{
	DithAnim_startPointImpl *startPoint = new DithAnim_startPointImpl(this, "Edit Point", anim[0]);
	startPoint->exec();
	if (startPoint->getExitStatus())
	{
		anim[0] = startPoint->getDithAnimStruct();
	}
	delete startPoint;
}



void DithAnimationEditorImpl::startAnimationButtonClicked()
{
	if (number > 1)
		exitStatus = true;
	close();
}