#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>


#include "FCAnim_newPointImpl.h"
#include "focusContextEditorImpl.h"


#define PI 3.14159265


FCAnim_newPointImpl::FCAnim_newPointImpl(QWidget *parent, const char *name, FCAnim_struct newPoint, FCAnim_struct oldPoint) : FCAnim_newPoint(parent, name)
{
	anim = newPoint;
	animRef = oldPoint;
	setValues();
	exitStatus = false;
}


FCAnim_newPointImpl::~FCAnim_newPointImpl()
{
}


FCAnim_struct FCAnim_newPointImpl::getFCAnimStruct()
{
	return anim;
}


bool FCAnim_newPointImpl::getExitStatus()
{
	return exitStatus;
}


bool FCAnim_newPointImpl::getValues()
{
	bool flag;
	anim.transfC = (FC_transferCoefficient->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.ksc = (FC_ksc->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.kss = (FC_kss->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.kse = (FC_kse->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.redOffset = (FC_redOffset->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.redFactor = (FC_redFactor->text()).toFloat(&flag);
	if (!flag)
		return false;
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	anim.xClockwise = xRotClockwise->isChecked();
	anim.yClockwise = yRotClockwise->isChecked();
	anim.zClockwise = zRotClockwise->isChecked();

	anim.samples = samplesSlider->value();

	return true;
}


void FCAnim_newPointImpl::setValues()
{
	// Line Edit
	FC_transferCoefficient->setText(QString::number(anim.transfC));
	FC_ksc->setText(QString::number(anim.ksc));
	FC_kss->setText(QString::number(anim.kss));
	FC_kse->setText(QString::number(anim.kse));
	FC_redOffset->setText(QString::number(anim.redOffset));
	FC_redFactor->setText(QString::number(anim.redFactor));

	// Slider
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));

	xRotClockwise->setChecked(anim.xClockwise);
	yRotClockwise->setChecked(anim.yClockwise);
	zRotClockwise->setChecked(anim.zClockwise);
	
	int diffX, diffY, diffZ;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;

	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;

	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;

	xRotLabel_diff->setText(QString::number(diffX));
	yRotLabel_diff->setText(QString::number(diffY));
	zRotLabel_diff->setText(QString::number(diffZ));

	samplesSlider->setValue(anim.samples);
	samplesLabel->setText(QString::number(anim.samples));
}


void FCAnim_newPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void FCAnim_newPointImpl::setFCRegionButtonClicked()
{
    float orientation [3];
	orientation[0] = (float)anim.xRot;
	orientation[1] = (float)anim.yRot;
	orientation[2] = (float)anim.zRot;
	focusContextEditorImpl *focConEditor = new focusContextEditorImpl(this, "Renderer - Set Focus Region", anim.dimension, orientation, (double *)anim.focPoint, anim.center, anim.radius);
    focConEditor->exec();
    if (focConEditor->okay)
    {
		float *middle;
		middle = focConEditor->getFocalPoint();
		anim.radius = focConEditor->getRadius();
		anim.center[0] = middle[0];
		anim.center[1] = middle[1];
		anim.center[2] = middle[2];
    }
	delete focConEditor;
}


void FCAnim_newPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void FCAnim_newPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void FCAnim_newPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}


void FCAnim_newPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}


void FCAnim_newPointImpl::samplePointsSliderChanged(int value)
{
	samplesLabel->setText(QString::number(value));
	anim.samples = value;
}


void FCAnim_newPointImpl::xRotClockwiseClicked(bool status)
{
	anim.xClockwise = status;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void FCAnim_newPointImpl::yRotClockwiseClicked(bool status)
{
	anim.yClockwise = status;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void FCAnim_newPointImpl::zRotClockwiseClicked(bool status)
{
	anim.zClockwise = status;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}