#ifndef MLANIM_NEWPOINTIMPL_h
#define MLANIM_NEWPOINTIMPL_h

#include <qwidget.h>

#include "MLAnim_newPoint.h"
#include "MLAnim_struct.h"



//! Class MLAnim_newPointImpl
/*!
    Diese Klasse ist abgeleitet von der Klasse MLAnim_newPoint. In ihr sind die 
	Methoden fr die Actions im Dialog ausprogrammiert.
*/
class MLAnim_newPointImpl : public MLAnim_newPoint
{
    Q_OBJECT;
    
public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen. newPoint ist eine Struktur, 
		welche die Werte des aktuellen Animations Punkts enth&auml;t. oldPoint enth&auml;lt die
		Werte des Vorg&auml;nger.
	*/
    MLAnim_newPointImpl(QWidget* parent, const char* name, MLAnim_struct newPoint, MLAnim_struct oldPoint);
    //! Destruktor
	~MLAnim_newPointImpl();
	//! Methode getExitStatus
	/*!
		Gibt true zur&uuml;ck, wenn der Dialog mit OK beendet wurde.
	*/
	bool getExitStatus();
	//! Methode getMLAnimStruct
	/*!
		Gibt die neuen Werte des Animationspunkts in einer Struktur zur&uuml;ck.
	*/
	MLAnim_struct getMLAnimStruct();
    
public slots:
    //! Slot xRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die x-Rotation bewegt wird.
	*/
    void xRotSliderChanged(int value);
	//! Slot yRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die y-Rotation bewegt wird.
	*/
	void yRotSliderChanged(int value);
	//! Slot zRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die z-Rotation bewegt wird.
	*/
	void zRotSliderChanged(int value);
	//! Slot zoomSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r den Zoom bewegt wird.
	*/
	void zoomSliderChanged(int value);
	//! Slot okayButtonClicked
	/*!
		Wird aufgerufen wenn der OK Button geclickt wird.
	*/
	void okayButtonClicked();
	//! Slot samplePointsSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Anzahl der Frames bewegt wird.
	*/
	void samplePointsSliderChanged(int value);
	//! Slot xRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die x-Achse
		geclickt wird.
	*/
	void xRotClockwiseClicked(bool status);
	//! Slot yRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die y-Achse
		geclickt wird.
	*/
	void yRotClockwiseClicked(bool status);
	//! Slot zRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die z-Achse
		geclickt wird.
	*/
	void zRotClockwiseClicked(bool status);
	//! Slot setMLButtonClicked
	/*!
		Wird aufgerufen wenn auf den Set Magic Lamp Button geclickt wird.
	*/
	void setMLButtonClicked();
	//! Slot angleDiffSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Differenz des Winkels ver&auml;ndert wird.
	*/
	void angleDiffSliderChanged(int value);
	//! Slot transferFSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r den Transfer-Koeffizienten ver&auml;ndert wird.
	*/
	void transferFSliderChanged(int value);
    
private:
	bool getValues();
	void setValues();

	MLAnim_struct anim;
	MLAnim_struct animRef;
	bool exitStatus;
};


#endif