#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <math.h>


#include "MLAnim_startPointImpl.h"
#include "MLDirectionEditorImpl.h"


#define PI 3.14159265


MLAnim_startPointImpl::MLAnim_startPointImpl(QWidget *parent, const char *name, MLAnim_struct point) : MLAnim_startPoint(parent, name)
{
	anim = point;
	setValues();
	exitStatus = false;
}


MLAnim_startPointImpl::~MLAnim_startPointImpl()
{
}


MLAnim_struct MLAnim_startPointImpl::getMLAnimStruct()
{
	return anim;
}


bool MLAnim_startPointImpl::getExitStatus()
{
	return exitStatus;
}


bool MLAnim_startPointImpl::getValues()
{
	anim.angle_diff = angleDiffSlider->value();
	anim.transfer = (float)transferFSlider->value()/100;
	anim.backside = backside->isChecked();
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	return true;
}


void MLAnim_startPointImpl::setValues()
{
	angleDiffSlider->setValue(anim.angle_diff);
	transferFSlider->setValue((int)(anim.transfer*100));
	angleDiffLabel->setText(QString::number(anim.angle_diff));
	transferFLabel->setText(QString::number(anim.transfer));
	backside->setChecked(anim.backside);

	// Slider
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));
}


void MLAnim_startPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void MLAnim_startPointImpl::setMLButtonClicked()
{
    MLDirectionEditorImpl *mlDirEditor = new MLDirectionEditorImpl(this, "MLDirEditor", anim.sideDir, anim.uprightDir);
	mlDirEditor->exec();
	if (mlDirEditor->getExitStatus())
	{
		float upright = mlDirEditor->getUprightDirection();
		float side = mlDirEditor->getSideDirection();
		anim.direction[2] = cos(upright*PI/180) * cos(side*PI/180);
		anim.direction[0] = (-1)*sin(upright*PI/180) * cos(side*PI/180);
		anim.direction[1] = (-1)*sin(side*PI/180);
		float length = sqrt(anim.direction[0]*anim.direction[0] + anim.direction[1]*anim.direction[1]+ anim.direction[2]*anim.direction[2]);
		anim.direction[0] /= length;
		anim.direction[1] /= length;
		anim.direction[2] /= length;
		anim.uprightDir = upright;
		anim.sideDir = side;
	}
	delete mlDirEditor;
}


void MLAnim_startPointImpl::angleDiffSliderChanged(int value)
{
	angleDiffLabel->setText(QString::number(value));
	anim.angle_diff = value;
}


void MLAnim_startPointImpl::transferFSliderChanged(int value)
{
	transferFLabel->setText(QString::number((float)value/100));
	anim.transfer = (float)value/100;
}


void MLAnim_startPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
}


void MLAnim_startPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
}


void MLAnim_startPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
}


void MLAnim_startPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}
