
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "vtkPointData.h"

#include "reader.h"



Reader::Reader(QString file, int dim [3], float space [3])
{
    flag = true;
    tf_factor = 1;
    filename = file;
	error_report = 0;
    
    dimension[0] = dim[0];
    dimension[1] = dim[1];
    dimension[2] = dim[2];
    
    spacing[0] = space[0];
    spacing[1] = space[1];
    spacing[2] = space[2];
    
	data = vtkImageData::New();

    if (filename.endsWith(".raw") || filename.endsWith(".RAW"))
    {
		readRaw();
		range = 256;
    }
    else
    {
		readDat();
		range = 4096;
    }
}


Reader::~Reader()
{
    //delete histogram;
	data->Delete();
}


vtkUnsignedCharArray* Reader::readRawData()
{
    unsigned char *array = new unsigned char [dimension[0]*dimension[1]*dimension[2]];
    FILE *file;
    long fileSize;
       
    file = fopen(filename, "rb");
       
    if (file == NULL)
    {
		error_report = 1;    // Could not open file
    }
    else
    {
		fseek(file, 0, SEEK_END);
		fileSize = ftell(file);
		rewind(file);
	   
		if (fileSize != dimension[0]*dimension[1]*dimension[2])
		{
			error_report = 2;    // file dimension and input dimension are not the same
		}
		else
		{
			fread(array, 1, fileSize, file);
			makeHistogramChar(array, dimension[0]*dimension[1]*dimension[2]);
		}
    }
    fclose(file);
    
    vtkUnsignedCharArray *dataArray = vtkUnsignedCharArray::New();
    dataArray->SetVoidArray(array, dimension[0]*dimension[1]*dimension[2], 1);
       
    return dataArray;
}


vtkUnsignedShortArray* Reader::readDatData()
{
    unsigned short *array = new unsigned short [dimension[0]*dimension[1]*dimension[2]];
    FILE *file;
    long fileSize;
       
    file = fopen(filename, "rb");
       
    if (file == NULL)
    {
		error_report = 1;   // could not open file
    }
    else
    {
		fseek(file, 0, SEEK_END);
		fileSize = ftell(file);
		rewind(file);
	
		if (fileSize == (dimension[0]*dimension[1]*dimension[2]*2 + 6))
		{
			fseek (file, 6, SEEK_SET);
			fileSize = fileSize - 6;
		}
	
		if (fileSize != dimension[0]*dimension[1]*dimension[2]*2)
		{
			error_report = 2;    // file dimension and input dimension are not the same
		}   
		else
		{
			fread(array, 1, fileSize, file);
			long max = findMax(array, dimension[0]*dimension[1]*dimension[2]);
			makeHistogramShort(array, dimension[0]*dimension[1]*dimension[2], max);
		}
    }
    fclose(file);
       
    vtkUnsignedShortArray *dataArray = vtkUnsignedShortArray::New();
    dataArray->SetArray(array, dimension[0]*dimension[1]*dimension[2], 1);
    
    return dataArray;
}


int Reader::getRange()
{
	return range;
}


void Reader::readRaw()
{
    data->GetPointData()->SetScalars(readRawData());
    data->SetDimensions(dimension);
    data->SetSpacing(spacing);
    data->SetScalarTypeToUnsignedChar();
}


void Reader::readDat()
{
    data->GetPointData()->SetScalars(readDatData());
    data->SetDimensions(dimension);
    data->SetSpacing(spacing);
    data->SetScalarTypeToUnsignedShort();
}


vtkImageData* Reader::getData()
{
    return data;
}


void Reader::getHistogram(unsigned int *histo)
{
    for (int i = 0; i < 256 ; i++)
		histo[i] = histogram[i];
}


double Reader::getFactor()
{
    return tf_factor;
}


int Reader::getErrorReport()
{
    return error_report;
}


long Reader::findMax(unsigned short *array, long length)
{
    long max = 0;
    long i;
    
    for (i = 0; i < length; i++)
    {
		if (array[i] > max)
		{
			max = array[i];
		}
    }    
    return max;
}


void Reader::makeHistogramShort(unsigned short *array, long length, long max_range)
{
    double factor1 = (double)256/(double)max_range;
    double factor2, max_value = 0;
    long i;
    int help;
    
    // set the factor for the transfer Functions
    tf_factor = factor1;
    
    for (i = 0; i < 256; i++)
    {
		histogram[i] = 0;
    }
    
    for (i = 0; i < length; i++)
    {
		help = int(array[i]*factor1);
		histogram[help]++;
		if (histogram[help] > max_value)
			max_value = histogram[help];
    }
    
    for (i = 0; i < 256; i++)
    {
		if (histogram[i] != 0)
		{
			factor2 = sqrt(max_value/histogram[i]);
			histogram[i] = (long)(histogram[i]*factor2);
		}
    }
    histogram[0] = 0;
}


void Reader::makeHistogramChar(unsigned char *array, long length)
{
    long i;
    double factor, max_value = 0;
    
    for (i = 0; i < 256; i++)
    {
		histogram[i] = 0;
    }
    
    for (i = 0; i < length; i++)
    {
		histogram[(int)array[i]]++;
		if (histogram[(int)array[i]] > max_value)
			max_value = histogram[(int)array[i]];
    }
    
    for (i = 0; i < 256; i++)
    {
		if (histogram[i] != 0)
		{
			factor = sqrt(max_value/histogram[i]);
			histogram[i] = (long)(histogram[i]*factor);
		}
    }
    histogram[0] = 0;
}
