#ifndef READER_H
#define READER_H

#include <qstring.h>

#include "vtkUnsignedCharArray.h"
#include "vtkUnsignedShortArray.h"
#include "vtkImageData.h"


//! Class Reader
/*!
    Diese Klasse l&auml;dt einen Datensatz aus einer Datei. Sie kann entweder DAT files
	mit 12 bit pro Datenpunkt laden oder RAW Files mit 8 bit pro Datenpunkt.
*/
class Reader
{  
public:
	//! Konstruktor
	/*!
		Setzt den Dateinamen und die Dimension und das Spacing der Datei.
	*/
    Reader(QString filename, int dim [3], float space [3]);
	//! Destruktor
    ~Reader();
	//! Methode getData
	/*!
		Gibt die eingelesenen vtkImageData zur&uuml;ck.
	*/
    vtkImageData* getData();
	//! Methode getHistogram
	/*!
		Setzt die Adresse eines Arrays auf das Hustogramm der Daten.
	*/
    void getHistogram(unsigned int *histo);
	//! Methode getFactor
	/*!
		Gibt den Faktor zur&uuml;ck, welcher angibt wie gross der Wertebereich eines Histogrammwertes ist.
	*/
    double getFactor();
	//! Methode getErrorReport
	/*!
		Gibt einen Integerwert zur&uuml;ck, welcher angibt wie der Lesezugriff verlaufen ist.
		Ist er 0 so war der Lesezugriff erfolgreich.
	*/
    int getErrorReport();
	//! Methode getRange
	/*!
		Gibt den Wertebereich der Daten zur&uuml;ck.
	*/
	int getRange();
    
    
	/*!
		Dieser boolsche Wert ist true, wenn der Lesezugriff erfolgreich war.
	*/
	bool flag;
    
private:
    void readRaw();
    void readDat();
    vtkUnsignedCharArray* readRawData();
    vtkUnsignedShortArray* readDatData();
    long findMax(unsigned short *array, long length);
    void makeHistogramShort(unsigned short * array, long length, long max);
    void makeHistogramChar(unsigned char * array, long length);
    
    vtkImageData *data;
    QString filename;
	int error_report;
    int dimension [3];
    float spacing [3];
    unsigned int histogram [256];
    double tf_factor;
	int range;
};

#endif
