
#include <math.h>
#include <iostream.h>
#include <qcolor.h>

#include "transferFunctionViewer.h"


#define PI 3.14159265


transferFunctionViewer::transferFunctionViewer(QWidget *parent, const char *name) : QWidget (parent, name)
{
//    histogram = new unsigned int [256];
	transFunc = new transferFunction();
    current = -1;
    int i;
    for (i = 0; i < 256; i++)
    {
		histogram[i] = 0;
    }
    this->setBackgroundColor(QColor(245, 235, 235));
    transFunc = NULL;
	windowing = false;
}


transferFunctionViewer::~transferFunctionViewer()
{
}


void transferFunctionViewer::setHistogram(unsigned int *histo)
{
	unsigned int max = 0;
	int i;
	double factor;
	for (i = 1; i < 256; i++)
	{
		if (max < histo[i])
			max = histo[i];
	}
	for (i = 0; i < 256; i++)
	{
		if (histo[i] != 0)
		{
			factor = sqrt(max/histo[i]);
			histogram[i] = (unsigned int)(histo[i]*factor);
		}
    }
}


void transferFunctionViewer::setWindowing(int begin, int end)
{
	windowing_begin = begin;
	windowing_end = end;
	windowing = true;
	update ();
}


void transferFunctionViewer::setTransferFunction(transferFunction *transFunction)
{
    transFunc = transFunction;
}


void transferFunctionViewer::setCurrent(int index)
{
    current = index;
    update ();
}


void transferFunctionViewer::paintHistogram(QPainter *paint)
{
    int i;
    unsigned int max = 0;
    double stepx = (double)this->width()/(double)256;
    QPen *pen = new QPen (QColor (0, 0, 0));
    paint->setPen (*pen);
    for (i = 1; i < 256; i++)
    {
		if (max < histogram[i])
			max = histogram[i];
    }
    histogram[0] = 0;
    double stepy = (double)this->height()/(double)(max * 1.1);
    double x = 0;
    int ymax = this->height() - 1;
    for (i = 1; i  < 256; i++)
	{	
		paint->drawLine ((int)(x), ymax-(int)(histogram[i-1]*stepy), (int)(x+stepx), ymax-(int)(histogram[i]*stepy));
		x = x + stepx;
    }
    delete pen;
}


void transferFunctionViewer::paintTransferFunctions(QPainter *paint)
{
    int i, x0, x1, y1, x2, y2, x3;
    int ymax = this->height();
    double factor = (double)this->width()/256;
    
    if(!transFunc)
	return;
    transfer_function tf = transFunc->tf;
    if(tf.element == NULL)
	cout<<"element is null\n";
    for (i = 0; i < tf.length; i ++)
    {
		QPen *pen = new QPen(tf.element[i].color);
		paint->setPen (*pen);
		if (i == current)
		{
			//QPen pen = paint->pen ();
			//QPen *pen = new QPen ();
			pen->setWidth (2);
			//pen.setWidth (2);
			paint->setPen (*pen);
		}
		x0 = (int)((tf.element[i].begin - 45*tf.element[i].opacity*tan(tf.element[i].angle_left*PI/180)) * factor);
		x1 = (int)((tf.element[i].begin) * factor);
		y1 = ymax - (int)(tf.element[i].opacity*0.9 * this->height());
		x2 = (int)((tf.element[i].begin + tf.element[i].length) * factor);
		y2 = y1;
		x3 = (int)(x2 + ((45*tf.element[i].opacity*tan (tf.element[i].angle_right*PI/180)) * factor));
		paint->drawLine(x0, ymax, x1, y1);
		paint->drawLine(x1, y1, x2, y2);
		paint->drawLine(x2, y2, x3, ymax);
		delete pen;
    }
}


void transferFunctionViewer::paintWindowing(QPainter *paint)
{
	QPen *pen = new QPen(QColor ("red"));
	paint->setPen (*pen);
	double factor = (double)this->width()/256;
	int ymax = this->height() - 1;
	paint->drawLine(0, ymax, (int)(windowing_begin*factor), ymax);
	paint->drawLine((int)(windowing_begin*factor), ymax, (int)(windowing_end*factor), 0);
	paint->drawLine((int)(windowing_end*factor), 0, 256, 0);
	delete pen;
}


void transferFunctionViewer::paintEvent(QPaintEvent * event)
{
    QPainter *paint = new QPainter(this);
	paintHistogram(paint);
    if(transFunc != NULL && !windowing)
	{
		if (transFunc->tf.length != 0)
			paintTransferFunctions (paint);
	}
	else if (windowing)
		paintWindowing(paint);
	delete paint;
}
