#ifndef TRANSFERFUNCTIONVIEWER_H
#define TRANSFERFUNCTIONVIEWER_H


#include <qwidget.h>
#include <qevent.h>
#include <qpainter.h>


#include "transferFunction.h"


//! Class transferFunctionViewer
/*!
    Diese Klasse ist eine Subklasse von QWidget. Sie erzeugt einen Widget in dem
	die Transfer Functions und das Histogram dargestellt werden.
*/
class transferFunctionViewer : public QWidget
{
    Q_OBJECT
    
public:
	//! Konstruktor
	/*!
		Setzt GUI Elemente.
	*/
    transferFunctionViewer(QWidget *parent, const char *name);
	//! Destruktor
    ~transferFunctionViewer();
	//! Methode setHistogram
	/*!
		Setzt das Histogram welches dann dargestellt wird.
	*/
    void setHistogram(unsigned int histo [256]);
	//! Methode setTransferFunction
	/*!
		Setzt die Transfer Functions welche dann dargestellt werden.
	*/
    void setTransferFunction(transferFunction *transFunction);
	//! Methode setWindowing
	/*!
		Setzt den Anfang und den Endwert von Windowing, falls Windowing gesetzt
		ist.
	*/
	void setWindowing(int begin, int end);
    
public slots:
	//! Methode setCurrent
	/*!
		Setzt die zur zeit markierte Transfer Function.
	*/
    void setCurrent(int index);
       
protected:
    void paintEvent(QPaintEvent *event);
    
private:
    void paintHistogram(QPainter *paint);
    void paintTransferFunctions(QPainter *paint);
	void paintWindowing(QPainter *paint);
	//virtual void setPaletteBackgroundColor (QColor);

    unsigned int histogram[256];
    int current;    
	bool windowing;
	int windowing_begin, windowing_end;
    transferFunction *transFunc;
    //QPainter *paint;
};

#endif // TRANSFERFUNCTIONVIEWER
