#ifndef VTKBOX_H
#define VTKBOX_H

#include <qwidget.h>
#include <qstring.h>

#include "vtkQtRenderWindow.h"
#include "vtkRenderer.h"
#include "vtkImageData.h"
#include "vtkPiecewiseFunction.h"
#include "vtkColorTransferFunction.h"
#include "vtkVolumeRayCastMapper.h"
#include "vtkVolumeProperty.h"

#include "VolumeRayCastCompositeFunction.h"
#include "FCAnim_struct.h"
#include "DithAnim_struct.h"
#include "MLAnim_struct.h"


//! Class VTKBox
/*!
    Diese Klasse ist abgeleitet von vtkQtRenderWindow. Hier ist der logische Ablauf des
	Programms definiert. Es wird auch das gesamte Rendering-Verfahren hier definiert.
*/
class VTKBox : public vtkQtRenderWindow 
{
    Q_OBJECT
    
public:
	//! Konstruktor
	/*!
		Konstruktor setzt parent-Widget und Widget-Namen. Es werden auch die Parameter
		und Varablen initialisiert.
	*/
    VTKBox(QWidget *parent, const  char *name);
	//! Destruktor
    ~VTKBox();
	//! Methode setRenderer
	/*!
		Diese Methode initialisiert den Renderer. Es wird ein Datensatz und eine Transfer
		Function &uuml;bergeben.
	*/
    void setRenderer(vtkImageData *imageData, vtkColorTransferFunction *colorFunc, vtkPiecewiseFunction *opacityFunc);
    //! Methode repaintTF
	/*!
		Die Parameter definieren eine Transfer Function. Bei Aufruf dieser Methode wird
		das Ergebnis mit den neuen Transfer Functions neu gerendert.
	*/
	void repaintTF(vtkColorTransferFunction *colorFunc, vtkPiecewiseFunction *opacityFunc);
    //! Methode getCamPos
	/*!
		Methode gibt die Position der Kamera zur&uuml;ck.
	*/
	double* getCamPos();
	//! Methode getFocalPoint
	/*!
		Methode gibt die Position des Fokuspunkts zurck.
	*/
    double* getFocalPoint();
	//! Methode getOrientation
	/*!
		Methode gibt die Orientierung des Datensatzes zur&uuml;ck.
	*/
	float* getOrientation();
	//! Methode setBackground
	/*!
		Diese Methode setzt die Farbe des Hintergrunds auch die &uuml;bergebene Farbe.
	*/
	void setBackground(float background [3]);
	//! Methode getBackground
	/*!
		Gibt die aktuelle Background-Farbe zur&uuml;ck.
	*/
	float * getBackground();
	//! Methode setFocalPoint
	/*!
		Diese Methode setzt den Fokuspunkt auf den Wert von focPoint.
	*/
    void setFocalPoint(double *focPoint);
	//! Methode saveImage
	/*!
		Diese Methode speichert den Inhalt des Rendering-Fensters in der Datei file ab.
	*/
    void saveImage(QString file, bool DOF, bool DOF_random);
	//! Methode startDOFAnimation
	/*!
		Diese Methode startet eine Animation f&uuml;r den Depth of Field Effekt. Die Parameter
		steuern die Animation.
	*/
    void startDOFAnimation(double points [20][3], int number, float start, float end, int frames, int samplePoints, bool fixedPoints, QString directory);
	//! Methode rendering
	/*!
		Setzt die Parameter der General-Settings.
	*/
	void rendering(int xRot, int yRot, int zRot, double zoom, float sampleStep, bool parallelProj);
	//! Methode rendering
	/*!
		Setzt die Parameter Gradient Opacity Function.
	*/
	void rendering(vtkPiecewiseFunction *gradientFunc);
	//! Methode setFocalDisk
	/*!
		Die Methode setzt die Dicke der Linse f&uuml;r den Depth of Field Effekt.
	*/
	void setFocalDisk(double focalDisk);
	//! Methode setSamplePoints
	/*!
		Die Methode setzt die Anzahl der Sample-Punkte f&uuml;r den Depth of Field Effekt.
	*/
    void setSamplePoints(int value);
	//! Methode renderingDOF
	/*!
		&Uuml;bergibt einen boolschen Wert, welcher bestimmt ob zuf&auml;llige Sample-Punkte
		verwendet werden sollen oder nicht.
	*/
	void renderingDOF(bool random);
	//! Methode renderingDOF
	/*!
		&Uuml;bergibt die Werte f&uuml;r die Linsendicke und die Anzahl der Sample-Punkte.
	*/
	void renderingDOF(double focalDisc, int samplePoints);
	//! Methode getGradientRange
	/*!
		Gibt den maximalen Wert zur&uuml;ck, welchen der gradient annehmen kann.
	*/
	float getGradientRange();
	//! Methode renderingFC
	/*!
		Setzt die Parameter f&uuml;r den Mittelpunkt und den Radius des Fokusbereich
	*/
	void renderingFC(float *middle, float radius);
	//! Methode renderingFC
	/*!
		Setzt die Parameter f&uuml;r den Kontextbereich beim Focus+Context
	*/
	void renderingFC(vtkColorTransferFunction *cTF, vtkPiecewiseFunction *pwF, float transC, float ksc, float kss, float kse, float redOffset, float redFactor);
	//! Methode setFocusContext
	/*!
		Schaltet die Focus+Context Methode ein oder aus.
	*/
	void setFocusContext(bool status);
	//! Methode startFCAnimation
	/*!
		Startet Animation f&uuml;r Focus+Context mit den &uuml;bergebenen Parametern.
	*/
	void startFCAnimation(FCAnim_struct *animPoints, int number, QString filename, bool status);
	//! Methode setDitheringParameters
	/*!
		Diese Methode setzt die Parameter f&uuml;r die 3D Dithering Methode.
	*/
	void setDitheringParameters(float distance, float size, bool context, bool structure, bool second_row, bool second_row_structure);
	//! Methode setDithering
	/*!
		Schaltet die 3D Dithering Methode ein oder aus.
	*/
	void setDithering(bool status);
	//! Methode setMagicLampParameters
	/*!
		Schaltet die 3D Dithering Methode ein oder aus.
	*/
	void setMagicLampParameters(float direction [3], float angle, float transfer_factor, bool backside);
	//! Methode setMagicLamp
	/*!
		Setzt die Parameter f&uuml;r die Magic Lamp Methode.
	*/
	void setMagicLamp(bool status);
	//! Methode startDithAnimation
	/*!
		Startet Animation f&uuml;r 3D Dithering mit den &uuml;bergebenen Parametern.
	*/
	void startDithAnimation(DithAnim_struct *animPoints, int number, QString filename, bool status);
	//! Methode startMLAnimation
	/*!
		Startet Animation f&uuml;r Magic Lamp mit den &uuml;bergebenen Parametern.
	*/
	void startMLAnimation(MLAnim_struct *animPoints, int number, QString filename, bool status);

	//! Methode RenderImage
	/*!
		Diese Methode rendert ein neues Ergebnisbild mit den derzeitigen Einstellungen.
	*/
	void RenderImage();
    
signals:
	void setRenderStatus(QString status);

protected:
    vtkRenderer* makeRenderer(vtkImageData *imageData, vtkColorTransferFunction *cTFun, vtkPiecewiseFunction *oTFun);
    
private:    
    vtkRenderer *Renderer;
	vtkVolume *volume;
	vtkVolumeRayCastMapper *volumeMapper;
	vtkVolumeProperty *volumeProperty;
	vtkRenderer *focConRenderer;
    int FDFrames;
	float gradientRange;
	bool right_rot;

	int dimension [3];
	float spacing [3];
	float ML_direction [3];
	float background [3];

	VolumeRayCastCompositeFunction *compositeFunction;
};

#endif  // VTKBOX
