#ifndef VIEWWIDGETIMPL_h
#define VIEWWIDGETIMPL_h

#include "vtkImageData.h"
#include "vtkColorTransferFunction.h"
#include "vtkPiecewiseFunction.h"

#include "viewWidget.h"
#include "transferFunction.h"


//! Class viewWidgetImpl
/*!
    Diese Klasse ist abgeleitet von viewWidget. Sie deiniert alle Funktionen und Abl&auml;ufe 
	des Hauptfenster. Im Hauptfenster werden alle Einstellungen vorgenommen.
*/
class viewWidgetImpl : public viewWidget
{
    Q_OBJECT    
    
 public:
	//! Konstruktor
	/*!
		Initialisierung der Parameter und Variablen. Setzen der Werte in den Eingabefeldern.
	*/
    viewWidgetImpl(QWidget *parent, const char *name);
	//! Destruktor
    ~viewWidgetImpl();
	//! Methode openEditor
	/*!
		&Ouml;ffnet ein neues Dialogfenster zum Einstellen der Transfer Functions.
	*/
    virtual void openEditor();
    
 public slots:

    //! Slot saveImage
	/*!
		Handelt die Action wenn im Men&uuml; auf Save Image geclickt wird.
	*/
	void saveImage();
	//! Slot openFile
	/*!
		Handelt die Action wenn im Men&uuml; auf Open File geclickt wird.
	*/
	void openFile();
	//! Slot DOFAnimation
	/*!
		Wird aufgerufen, wenn auf Depth of Field Animation geclickt wird.
	*/
    void DOFAnimation();
	//! Slot FCAnimation
	/*!
		Wird aufgerufen, wenn auf Focus+Context Animation geclickt wird.
	*/
	void FCAnimation();
	//! Slot DithAnimation
	/*!
		Wird aufgerufen, wenn auf Dithering Animation geclickt wird.
	*/
	void DithAnimation();
	//! Slot MLAnimation
	/*!
		Wird aufgerufen, wenn auf Magic Lamp Animation geclickt wird.
	*/
	void MLAnimation();
	//! Slot renewValues
	/*!
		Methode erneuert die Werte in den Eingabefeldern.
	*/
    void renewValues();
	//! Slot setFocusButtonPressed
	/*!
		Wird aufgerufen wenn auf den Fokus-Punkt-Setzen Button geclickt wird.
	*/
    void setFocusButtonPressed();


	//! Slot setRenderStatus
	/*!
		Diese Methode setzt einen Text in die Statuszeile.
	*/
	void setRenderStatus(QString status);
	//! Slot setFileStatus
	/*!
		Diese Methode setzt einen Text in die Statuszeile.
	*/
	void setFileStatus(QString status);

	//! Slot parallelViewClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r parallele Projektion angeclickt wird.
	*/
	void parallelViewClicked(bool status);
	//! Slot perspectiveViewClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r perspektivische Projektion angeclickt wird.
	*/
	void perspectiveViewClicked(bool status);


	//! Slot xRotationChanged
	/*!
		Wird aufgerufen wenn Slider f&uuml;r x-Rotation bewegt wird.
	*/
	void xRotationChanged(int degrees);
	//! Slot yRotationChanged
	/*!
		Wird aufgerufen wenn Slider f&uuml;r y-Rotation bewegt wird.
	*/
	void yRotationChanged(int degrees);
	//! Slot zRotationChanged
	/*!
		Wird aufgerufen wenn Slider f&uuml;r z-Rotation bewegt wird.
	*/
	void zRotationChanged(int degrees);
	//! Slot zoomChanged
	/*!
		Wird aufgerufen wenn Slider f&uuml;r Zoom bewegt wird.
	*/
	void zoomChanged(int value);
	//! Slot backgroundColorButtonClicked
	/*!
		Diese Methode wird bei einem Click auf den Set Background Color Button aufgerufen.
	*/
	void backgroundColorButtonClicked();
	

	//! Slot renderButtonClicked
	/*!
		Diese Methode wird aufgerufen, wenn auf den Render Button geclickt wird.
	*/
	void renderButtonClicked();


	//! Slot lenseSliderChanged
	/*!
		Diese Methode wird aufgerufen, wenn der Slider f&uuml;r die Linsendicke bewegt wird.
	*/
	void lenseSliderChanged(int value);
	//! Slot samplePointsSliderChanged
	/*!
		Diese Methode wird aufgerufen, wenn der Slider f&uuml;r die Anzahl der Sample-Punkte beim Depth of Field bewegt wird.
	*/
	void samplePointsSliderChanged(int value);

	
	//! Slot setFocusContextButtonClicked
	/*!
		Wird aufgerufen, wenn der Button zum einstellen des Focus+Context Bereichs gedr&uuml;ckt wird.
	*/
	void setFocusContextButtonClicked();

	
	//! Slot loadGFButtonClicked
	/*!
		Wird aufgerufen, wenn auf den Button zum Laden der Gradient Opacity Function geclickt wird.
	*/
	void loadGFButtonClicked();
	//! Slot saveGFButtonClicked
	/*!
		Wird aufgerufen, wenn auf den Button zum Speichern der Gradient Opacity Function geclickt wird.
	*/
	void saveGFButtonClicked();

	
	//! Slot loadFCButtonClicked
	/*!
		Wird aufgerufen, wenn auf den Button zum Laden der Gocus+Context Einstellungen geclickt wird.
	*/
	void loadFCButtonClicked();
	//! Slot saveFCButtonClicked
	/*!
		Wird aufgerufen, wenn auf den Button zum Speichern der Gocus+Context Einstellungen geclickt wird.
	*/
	void saveFCButtonClicked();

	
	//! Slot setDOF
	/*!
		Wird aufgerufen, wenn die Check Box f&uuml den Depth of Field Effekt geclickt wird.
	*/
	void setDOF(bool status);
	//! Slot setFC
	/*!
		Wird aufgerufen, wenn die Check Box f&uuml den Focus+Context Effekt geclickt wird.
	*/
	void setFC(bool status);
	//! Slot setDith
	/*!
		Wird aufgerufen, wenn die Check Box f&uuml den 3D Dithering Effekt geclickt wird.
	*/
	void setDith(bool status);
	//! Slot setML
	/*!
		Wird aufgerufen, wenn die Check Box f&uuml den Magic Lamp Effekt geclickt wird.
	*/
	void setML(bool status);

	
	//! Slot contextColorButtonClicked
	/*!
		Wird aufgerufen, wenn der Button f&uuml;r die Context Farbe geclickt wird.
	*/
	void contextColorButtonClicked();

	
	//! Slot DithDistanceSliderChanged
	/*!
		Methode wird bei Ver&auml;nderung des Distance Slider beim Dithering aufgerufen.
	*/
	void DithDistanceSliderChanged(int value);
	//! Slot DithSizeSliderChanged
	/*!
		Methode wird bei Ver&auml;nderung des Size Slider beim Dithering aufgerufen.
	*/
	void DithSizeSliderChanged(int value);
	//! Slot DithSecondRowClicked
	/*!
		Methode wird aufgerufen, wenn auf die Check Box f&uuml;r die zweite Reihe beim Dithering geclickt wird.
	*/
	void DithSecondRowClicked(bool value);

	
	//! Slot MLDirectionButtonClicked
	/*!
		Wird aufgerufen, wenn auf den Button zum Setzen der Richtung f&uuml;r die Magic Lamp gedrckt wird.
	*/
	void MLDirectionButtonClicked();
	//! Slot MLAngleSliderChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r die Differenz der Winkel bei dem Magic Lamp Effekt ver&auml;ndert wird.
	*/
	void MLAngleSliderChanged(int value);
	//! Slot MLTransferSliderChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r den Transferparameter bei dem Magic Lamp Effekt ver&auml;ndert wird.
	*/
	void MLTransferSliderChanged(int value);
    
  private:
    void loaded(vtkImageData *imageData);
    void readDimension(QString filename);
    void readSpacing(QString filename);
    void defaultValues();
    void setEnabledWidgets(bool value);
	void setFCValues(QColor color, float transC, float ksc, float kss, float kse, float redOffset, float redFactor);
	bool getFCValues();
     
    QString tfFileName;
	QString goFileName;
	QString fcFileName;
    unsigned int histo [256];
    bool file_loaded;
    int dimension [3];
    float spacing [3];
    transferFunction *transFunc;
    vtkColorTransferFunction* colorFunction();
    vtkPiecewiseFunction* opacityFunction();

	// Statusvariablen
	double factor;
	float sampleDistance;
	bool parallelViewStatus, perspectiveViewStatus;
	int xstatus, ystatus, zstatus;
	double zoom;
    bool interactive;
    int xangle, yangle, zangle;
    double zoomvalue;
	int range;
	float background [3];

	bool status_changed;
	bool background_changed;

	bool FC_changes;
	bool DOF_changes;
	bool Dith_changes;
	bool ML_changes;
	bool DOF_status;
	bool FC_status;
	bool Dith_status;
	bool ML_status;

	// DOF
	bool DOF_randomSP_status;

	// focus + context
	float FC_middle [3];
	float FC_radius;
	QColor FC_color_status;
	float FC_transferCoefficient_status, FC_ksc_status, FC_kss_status, FC_kse_status;
	float FC_redOffset_status, FC_redFactor_status;

	// Dithering Statusvalues
	float dith_distance, dith_size;
	bool dith_context, dith_structure, dith_second_row, dith_second_row_structure;

	// Magic Lamp Status values
	float ML_direction [3];
	float ML_angle, ML_transferF;
	bool ML_backside;
	int ML_uprightDir, ML_sideDir;
};

#endif
